#define PROCESSING_COLOR_SHADER

const float infinity = 1.0/0.0;

uniform vec2 pixelDimensions;
uniform int maxIterations;
uniform float wh, xMin, yMin;

uniform int numNodes;
uniform float processingOffset;
uniform float nodeX[32];
uniform float nodeR[32];
uniform float nodeG[32];
uniform float nodeB[32];

void main(void) {
	// Normalized pixel position to complex plane position
	float maxPwh = max(pixelDimensions.x, pixelDimensions.y);
	float x = (gl_FragCoord.x/maxPwh)*wh+xMin;
	float y = (gl_FragCoord.y/maxPwh)*wh+yMin;
	
	// Complex plane window offsets for pixel windows that are not square
	float halfDelta = wh/maxPwh*0.5;
	x -= min((pixelDimensions.x-pixelDimensions.y)*halfDelta, 0.0);
	y -= min((pixelDimensions.y-pixelDimensions.x)*halfDelta, 0.0);
	
	// Mandelbrot Set code
	int i = 0;
	float zr = x;
	float zi = y;
	for (; i < maxIterations; i++) {
		float sqZr = zr*zr;
		float sqZi = zi*zi;
		float twoZri = 2.0*zr*zi;
		zr = sqZr-sqZi+x;
		zi = twoZri+y;
		if (sqZr+sqZi > 16.0) break;
	}
	
	if (i == maxIterations) {
		int lastIndex = numNodes-1;
		gl_FragColor = vec4(nodeR[lastIndex], nodeG[lastIndex], nodeB[lastIndex], 1.0);
	}
	else {
		// Map i to the color node bar
		float fi = max(float(i)+1.0-log(log(sqrt(zr*zr+zi*zi)))/log(2.0), 1.0);
		float mapI = (log(fi)/log(float(maxIterations)))*(pixelDimensions.x-processingOffset)+processingOffset;
	
		// Determine the left and right node indices relative to mapI
		int leftIndex = 0;
		int rightIndex = numNodes-1;
		float leftClosest = infinity;
		float rightClosest = infinity;
		for (int i = 0; i < numNodes; i++) {
			float leftDist = mapI-nodeX[i];
			float rightDist = nodeX[i]-mapI;
			if (leftDist > 0.0 && leftDist < leftClosest) {
				leftIndex = i;
				leftClosest = leftDist;
			}
			if (rightDist > 0.0 && rightDist < rightClosest) {
				rightIndex = i;
				rightClosest = rightDist;
			}
		}
	
		// Interpolate unless mapI < all x, mapI > all x, or there was a tie
		if (leftIndex != rightIndex) {
			// Assign parametric t values
			float t = (mapI-nodeX[leftIndex])/(nodeX[rightIndex]-nodeX[leftIndex]);
			float it = 1.0-t;
		
			// Interpolate among left and right node indices
			gl_FragColor = vec4(
				((nodeR[leftIndex]*it)+(nodeR[rightIndex]*t)), 
				((nodeG[leftIndex]*it)+(nodeG[rightIndex]*t)), 
				((nodeB[leftIndex]*it)+(nodeB[rightIndex]*t)), 
			1.0);
		}
		else gl_FragColor = vec4(nodeR[leftIndex], nodeG[leftIndex], nodeB[leftIndex], 1.0);
	}
}
